#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <misc.h>
#include "pppdialin.h"
#include "pppdialin.m"
#include "../../paths.h"

static HELP_FILE help_ipup ("pppdialin","ip-up");
extern HELP_FILE help_dialin;

static const char ppp_dialout[]="dialout";
static const char ppppostcon[]="/usr/lib/linuxconf/lib/ppppostlogin";
static const char ppppostdis[]="/usr/lib/linuxconf/lib/ppppostlogout";

static CONFIG_FILE f_ipup (ETC_PPP_IPUP
	,help_ipup
	,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
	,"root","root",0700,ppp_dialout);
static CONFIG_FILE f_ipdown (ETC_PPP_IPDOWN
	,help_ipup
	,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
	,"root","root",0700,ppp_dialout);

/*
	Make sure the config file has one call to the script
*/
static bool ipupupd_checkone(
	CONFIG_FILE &cf,
	const char *script,
	bool doit)
{
	bool ret = false;
	FILE *fin = cf.fopen ("r");
	if (fin != NULL){
		char buf[1000];
		int len = strlen(script);
		bool first = true;
		bool copy_sh = false;
		while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
			strip_end (buf);
			if (first){
				if (strcmp(buf,"#!/bin/sh")==0 
					|| strcmp(buf,"#!/bin/bash")==0){
					copy_sh = true;
				}
				first = false;
			}
			if (strncmp(buf,script,len)==0 && isspace(buf[len])){
				ret = true;
				break;
			}
		}
		if (!ret && doit){
			/* #Specification: pppdialin / updating ip-up ip-down
				We must register a script in that file, but we can't
				put it at the end because some scripts ends with "exit 0".
				So we put it at the beginning, after the #!/bin/sh if
				there.
			*/
			const char *cfpath = cf.getpath();
			char pathnew[PATH_MAX];
			snprintf (pathnew,sizeof(pathnew)-1,"%s.NEW",cfpath);
			FILE *fout = cf.fopen (pathnew,"w");
			if (fout != NULL){
				rewind (fin);
				if (copy_sh){
					if (fgets(buf,sizeof(buf)-1,fin)!=NULL) fputs (buf,fout);
				}
				fprintf (fout,"%s $*\n",script);
				while (fgets(buf,sizeof(buf)-1,fin)!=NULL) fputs (buf,fout);
				if (fclose (fout)!=-1){
					char pathbak[PATH_MAX];
					snprintf (pathbak,sizeof(pathbak)-1,"%s.BAK",cfpath);
					rename (cfpath,pathbak);
					rename (pathnew,cfpath);
				}
			}
		}
		fclose (fin);
	}else if (doit){
		FILE *fout = cf.fopen ("w");
		if (fout != NULL){
			fprintf (fout,"#!/bin/sh\n");
			fprintf (fout,"%s $*\n",script);
			fclose (fout);
		}
	}
	return ret;
}

void ipupupd_check()
{
	if (!ipupupd_checkone (f_ipup,ppppostcon,false)
		|| !ipupupd_checkone (f_ipdown,ppppostdis,false)){
		if (xconf_yesno (MSG_U(T_MODIPUP,"Modify ip-up/ip-down")
			,MSG_U(I_MODIPUP
				,"The files /etc/ppp/ip-up and /etc/ppp/ip-down\n"
				 "must be modified to support the pppdialin module\n"
				 "\n"
				 "The following lines must be added respectivly to\n"
				 "each file:\n"
				 "\n"
				 "ip-up  : /usr/lib/linuxconf/lib/ppppostlogin $*\n"
				 "ip-down: /usr/lib/linuxconf/lib/ppppostlogout $*\n"
				 "\n"
				 "Do you want to do it ?")
			,help_dialin)==MENU_YES){
			ipupupd_checkone (f_ipup,ppppostcon,true);
			ipupupd_checkone (f_ipdown,ppppostdis,true);
		}
	}
}

